class Animation2D {
	spritesheet: HTMLImageElement;
	x: number;
	y: number;
	width: number;
	height: number;
	frames: number;
	currentFrame: number;
	context: CanvasRenderingContext2D;
	
	constructor(spritesheet: HTMLImageElement,
				x: number,
				y: number,
				width: number,
				height: number,
				frames: number,
				context: CanvasRenderingContext2D) {
		this.spritesheet = spritesheet;
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
		this.context = context;
		this.frames = frames;
		this.currentFrame = 0;
	}
	
	getCurrentFrame() {
		return this.currentFrame;
	}
	
	getFrameCount() {
		return this.frames;
	}
	
	getHeight() {
		return this.height;
	}
	
	getSpritesheet() {
		return this.spritesheet;
	}
	
	getWidth() {
		return this.width;
	}
	
	getX() {
		return this.x;
	}
	
	getY() {
		return this.y;
	}
	
	maintain(fps: number) {
		var spritesheet = this.spritesheet,
			x = this.x,
			y = this.y,
			width = this.width,
			height = this.height,
			context = this.context,
			frames = this.frames,
			currentFrame = this.currentFrame;
		
		context.drawImage(spritesheet, 0, 0, width, height, x, y, width, height);
		
		function u() {
			return function() {
				context.clearRect(x, y, width, height);
				context.drawImage(
					spritesheet,
					0 + (width * currentFrame), 0, width, height,
					x, y, width, height
				);
				currentFrame++;
				if (currentFrame >= frames) {
					currentFrame = 0;
				}
			}
		}
			
		setInterval(u(), (1000 / fps));
	}
	
	setCurrentFrame(currentFrame: number) {
		this.currentFrame = currentFrame;
	}
}

class ControlScheme {
	scheme: Array<Array<any>>;

	constructor(scheme: Array<Array<any>>) {
		this.scheme = scheme;
	}

	getInputCode(key: string): number {
		for (let v of this.scheme) if (v[0] === key) return v[1];
	}
	
	getInputKey(code: number): string {
		for (let v of this.scheme) if (v[1] === code) return v[0];
	}
}

class Image2D {
	constructor(file: string) {
		let image = new Image();
		image.src = file;
		return image;
	}
}